/*
 * com/ngon/gfx/ZoomPixelSurface.java
 * Victor Liu > mailto:victor@n-gon.com
 *
 * Copyright (C) 2004 Victor Liu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.ngon.gfx;

/*
 * class ZoomPixelSurface
 */
public class ZoomPixelSurface extends PixelSurface {
    
    protected int zoom;
    
    public ZoomPixelSurface(int width, int height) {
        super(width, height);
        this.zoom = 1;
    }
    
    public ZoomPixelSurface(int width, int height, int zoom) {
        super(width, height);
        this.zoom = zoom;
    }
    
    public void setZoom(int zoom) {
        this.zoom = zoom;
    }
    
    public void paintField(int dstX, int dstY, int[] rgbField, 
                           int fieldWidth, int fieldHeight) {
        int p = 0;
        for (int y0 = dstY; y0 < dstY + fieldHeight*zoom; y0 += zoom) {
            for (int x0 = dstX; x0 < dstX + fieldWidth*zoom; x0 += zoom, p++) {
                int rgb = rgbField[p];
                for (int y = y0; y < y0 + zoom; y++) {
                    for (int x = x0; x < x0 + zoom; x++) {
                        PixelDrawer.setPixel(x, y, rgb, 
                                             pixels, width, height);
                    }
                }
            }
        }
    }
    
    public void paintRect(int srcX, int srcY, int srcW, int srcH, 
                          int[] rgbField, int fieldW, int fieldH,
                          int alpha,
                          int dstX, int dstY) {
        if (alpha == 0) {
            return;
        }
        int p = srcX + srcY * fieldW;
        int n = fieldW * fieldH;
        for (int y0 = dstY; 
             y0 < dstY + srcH*zoom;
             y0 += zoom, p += fieldW - srcW) {
            for (int x0 = dstX;
                 x0 < dstX + srcW*zoom;
                 x0 += zoom, p++) {
                if (p < 0) {
                    continue;
                } else if (p >= n) {
                    return;
                }
                int rgb = rgbField[p];
                if (alpha == 255) {
                    for (int y = y0; y < y0 + zoom; y++) {
                        for (int x = x0; x < x0 + zoom; x++) {
                            PixelDrawer.setPixel(x, y, rgb, 
                                                 pixels, width, height);
                        }
                    }
                } else {
                    for (int y = y0; y < y0 + zoom; y++) {
                        for (int x = x0; x < x0 + zoom; x++) {
                            PixelDrawer.mixPixel(x, y, rgb, alpha,
                                                 pixels, width, height);
                        }
                    }
                }
            }
        }
    }
}
