/*
 * com/ngon/sql/QueryRunner.java
 * Victor Liu > mailto:victor@n-gon.com
 *
 * Copyright (C) 2004 Victor Liu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.ngon.sql;

import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

/**
 * class QueryRunner
 */
final public class QueryRunner extends Thread {
    
    private String query;
    private Statement stmt;
    private ResultSet rs;
    
    public QueryRunner() {
        setDaemon(true);
    }
    
    public synchronized void begin(Statement stmt) {
        this.stmt = stmt;
        this.rs = null;
        notify();
    }
    
    public synchronized void begin(Statement stmt, String query) {
        this.query = query;
        this.begin(stmt);
    }
    
    public void run() {
        Thread currentThread = Thread.currentThread();
        while (this == currentThread) {
            try {
                if (stmt == null) {
                    synchronized(this) {
                        while (stmt == null) {
                            wait();
                        }
                    }
                }
            } catch (InterruptedException e) {}
            
            if (this != currentThread) {
                return;
            }
            
            try {
                if (stmt instanceof PreparedStatement) {
                    rs = ((PreparedStatement) stmt).executeQuery();
                } else {
                    rs = stmt.executeQuery(query);
                }
            } catch (SQLException e) {
                System.err.println(e);
            }
            
            reset();
        }
    }
    
    public synchronized ResultSet getResultSet() {
        return rs;
    }
    
    public synchronized void reset() {
        this.stmt = null;
        this.query = null;
    }
}
